#!/usr/local/bin/perl

# Script to create 125000 C functions that don't do much
# These functions then get profiled, and we see what happens then.
#

open (MAKEFILE, "> Makefile");
open (MAIN, "> main.cc");
open (MAININC, "> main.h");

print MAIN "#include \"main.h\"\n";
print MAIN "int main() {\n";

# 25 top level
for $primary (a..x,z) {
  mkdir $primary,0755;
  print MAKEFILE "PRIMARYLIST+=$primary.all\n";
  print MAKEFILE "LOADLIBES += -L$primary\n";
  print MAKEFILE "LDLIBS += -l$primary$primary\n";   
  chdir $primary;
  createPrimary($primary);  

  # 50 secondary
  for $secondary (1..50) {
    $secondName=$primary."_".$secondary;
    mkdir $secondName,0755;
    addSecondary($secondName);
    chdir $secondName;
    createSecondary($secondName);

    # 100 final
    for $routine (1..100) {
      $filename = $primary."_".$secondary."_".$routine;
      createFile($filename);
    }
    finalizeSecondary();
    chdir "..";
  }
  finalizePrimary();
  chdir "..";
  `cat Makefile.primary >> $primary/Makefile`;
}
close MAKEFILE;
`cat Makefile.master >> Makefile`;
print MAIN "}\n";
close MAIN;

sub createPrimary($) {
  my ($primary) = @_;
  open (PRIMARY, "> $primary.cc");
  open (PRIMARYINC, "> $primary.h");
  open (PRIMARYMAKE, "> Makefile");
  print PRIMARY "/* $primary.cc */\n";
  print PRIMARY "#include \"$primary.h\"\n";
  print PRIMARY "void $primary() {\n";
  print PRIMARYMAKE "PRIMARY = $primary\n";
  print MAIN "  $primary();\n";
  print MAININC "void $primary();\n";
}

sub finalizePrimary() {
  print PRIMARY "}\n";
  close PRIMARY;
  close PRIMARYINC;
  close PRIMARYMAKE;
}

sub addSecondary($) {
  my ($secondName) = @_;
  print PRIMARY "  $secondName();\n";
  print PRIMARYINC "void $secondName();\n";
}

sub createSecondary($) {
  my ($secondName) = @_;
  $secondGroup = $secondName."_group";
  open (SECONDFUNC, "> $secondName.cc");
  open (SECONDINC, "> $secondName.h");
  open (SECONDGROUP, "> $secondGroup.cc");
  print SECONDFUNC "/* $secondName.cc */\n";
  print SECONDFUNC "#include \"$secondName.h\"\n";
  print SECONDFUNC "void $secondName() {\n";
  print SECONDGROUP "#include \"$secondName.h\"\n";
  print PRIMARYMAKE "VPATH +=:$secondName\n";
  print PRIMARYMAKE "SECLIST += $secondName.all\n";
  print PRIMARYMAKE "$secondName.all : lib$primary$primary.a($secondName.o)\n";
  print PRIMARYMAKE "$secondName.all : lib$primary$primary.a($secondGroup.o)\n";
}

sub finalizeSecondary() {
  print SECONDFUNC "}\n";
  close SECONDFUNC;
  close SECONDGROUP;
  close SECONDINC;
}

sub createFile($) {
  my($filename) =@_;
  print SECONDGROUP "/* $filename.cc */\n";
  print SECONDGROUP "void $filename() {\n";
  print SECONDGROUP "  int i;\n";
  print SECONDGROUP "  for (i=0;i<10;i++);\n";
  print SECONDGROUP "}\n";
  print SECONDFUNC "  $filename();\n";
  print SECONDINC "void $filename();\n";
}
